from copy import copy

from .platforms import PLATFORMS

# Possible values to extract from a Git Url
REQUIRED_ATTRIBUTES = (
    "domain",
    "repo",
)


class GitUrlParsed:
    platform = None

    def __init__(self, parsed_info):
        self._parsed = parsed_info

        # Set parsed objects as attributes
        for k, v in parsed_info.items():
            setattr(self, k, v)

        for name, platform in PLATFORMS:
            if name == self.platform:
                self._platform_obj = platform
                break

    def _valid_attrs(self):
        return all([getattr(self, attr, None) for attr in REQUIRED_ATTRIBUTES])  # NOQA

    @property
    def valid(self):
        return all(
            [
                self._valid_attrs(),
            ]
        )

    ##
    # Alias properties
    ##
    @property
    def host(self):
        return self.domain

    @property
    def resource(self):
        return self.domain

    @property
    def name(self):
        return self.repo

    @property
    def user(self):
        if hasattr(self, "_user"):
            return self._user

        return self.owner

    @property
    def groups(self):
        if self.groups_path:
            return self.groups_path.split("/")
        else:
            return []

    def format(self, protocol):  # noqa : A0003
        """Reformat URL to protocol."""
        items = copy(self._parsed)
        items["port_slash"] = "%s/" % self.port if self.port else ""
        items["groups_slash"] = "%s/" % self.groups_path if self.groups_path else ""
        return self._platform_obj.FORMATS[protocol] % items

    @property
    def normalized(self):
        """Normalize URL."""
        return self.format(self.protocol)

    ##
    # Rewriting
    ##
    @property
    def url2ssh(self):
        return self.format("ssh")

    @property
    def url2http(self):
        return self.format("http")

    @property
    def url2https(self):
        return self.format("https")

    @property
    def url2git(self):
        return self.format("git")

    # All supported Urls for a repo
    @property
    def urls(self):
        return {protocol: self.format(protocol) for protocol in self._platform_obj.PROTOCOLS}

    ##
    # Platforms
    ##
    @property
    def github(self):
        return self.platform == "github"

    @property
    def bitbucket(self):
        return self.platform == "bitbucket"

    @property
    def friendcode(self):
        return self.platform == "friendcode"

    @property
    def assembla(self):
        return self.platform == "assembla"

    @property
    def gitlab(self):
        return self.platform == "gitlab"

    ##
    # Get data as dict
    ##
    @property
    def data(self):
        return dict(self._parsed)



from .base import BasePlatform


class GitHubPlatform(BasePlatform):
    PATTERNS = {
        "https": (
            r"(?P<protocols>(git\+)?(?P<protocol>https))://(?P<domain>[^/]+?)"
            r"(?P<pathname>/(?P<owner>[^/]+?)/(?P<repo>[^/]+?)(?:\.git)?(?P<path_raw>(/blob/|/tree/).+)?)$"
        ),
        "ssh": (
            r"(?P<protocols>(git\+)?(?P<protocol>ssh))?(://)?git@(?P<domain>.+?)(?P<pathname>(:|/)"
            r"(?P<owner>[^/]+)/(?P<repo>[^/]+?)(?:\.git)"
            r"(?P<path_raw>(/blob/|/tree/).+)?)$"
        ),
        "git": (
            r"(?P<protocols>(?P<protocol>git))://(?P<domain>.+?)"
            r"(?P<pathname>/(?P<owner>[^/]+)/(?P<repo>[^/]+?)(?:\.git)?"
            r"(?P<path_raw>(/blob/|/tree/).+)?)$"
        ),
    }
    FORMATS = {
        "https": r"https://%(domain)s/%(owner)s/%(repo)s.git%(path_raw)s",
        "ssh": r"git@%(domain)s:%(owner)s/%(repo)s.git%(path_raw)s",
        "git": r"git://%(domain)s/%(owner)s/%(repo)s.git%(path_raw)s",
    }
    DOMAINS = (
        "github.com",
        "gist.github.com",
    )
    DEFAULTS = {"_user": "git"}

    @staticmethod
    def clean_data(data):
        data = BasePlatform.clean_data(data)
        if data["path_raw"].startswith("/blob/"):
            data["path"] = data["path_raw"].replace("/blob/", "")
        if data["path_raw"].startswith("/tree/"):
            data["branch"] = data["path_raw"].replace("/tree/", "")
        return data



import unittest

from giturlparse import parse

# Test data
REWRITE_URLS = (
    # GitHub SSH
    ("git@github.com:Org/Repo.git", "ssh", "git@github.com:Org/Repo.git"),
    ("git@github.com:Org/Repo.git", "https", "https://github.com/Org/Repo.git"),
    ("git@github.com:Org/Repo.git", "git", "git://github.com/Org/Repo.git"),
    # GitHub HTTPS
    ("https://github.com/Org/Repo.git", "ssh", "git@github.com:Org/Repo.git"),
    ("https://github.com/Org/Repo.git", "https", "https://github.com/Org/Repo.git"),
    ("https://github.com/Org/Repo.git", "git", "git://github.com/Org/Repo.git"),
    # GitHub GIT
    ("git://github.com/Org/Repo.git", "ssh", "git@github.com:Org/Repo.git"),
    ("git://github.com/Org/Repo.git", "https", "https://github.com/Org/Repo.git"),
    ("git://github.com/Org/Repo.git", "git", "git://github.com/Org/Repo.git"),
    (
        "git://github.com/Org/Repo/blob/master/dir/subdir/path",
        "git",
        "git://github.com/Org/Repo.git/blob/master/dir/subdir/path",
    ),
    # BitBucket SSH
    ("git@bitbucket.org:Org/Repo.git", "ssh", "git@bitbucket.org:Org/Repo.git"),
    ("git@bitbucket.org:Org/Repo.git", "https", "https://Org@bitbucket.org/Org/Repo.git"),
    # BitBucket HTTPS
    ("https://Org@bitbucket.org/Org/Repo.git", "ssh", "git@bitbucket.org:Org/Repo.git"),
    ("https://Org@bitbucket.org/Org/Repo.git", "https", "https://Org@bitbucket.org/Org/Repo.git"),
    # Assembla GIT
    ("git://git.assembla.com/SomeRepoID.git", "ssh", "git@git.assembla.com:SomeRepoID.git"),
    ("git://git.assembla.com/SomeRepoID.git", "git", "git://git.assembla.com/SomeRepoID.git"),
    # Assembla SSH
    ("git@git.assembla.com:SomeRepoID.git", "ssh", "git@git.assembla.com:SomeRepoID.git"),
    ("git@git.assembla.com:SomeRepoID.git", "git", "git://git.assembla.com/SomeRepoID.git"),
    # FriendCode HTTPS
    ("https://friendco.de/Aaron@user/test-repo.git", "https", "https://friendco.de/Aaron@user/test-repo.git"),
    # Gitlab SSH
    ("git@host.org:Org/Repo.git", "ssh", "git@host.org:Org/Repo.git"),
    ("git@host.org:9999/Org/Repo.git", "ssh", "git@host.org:9999/Org/Repo.git"),
    ("git@host.org:Org/Repo.git", "https", "https://host.org/Org/Repo.git"),
    ("git@host.org:9999/Org/Repo.git", "https", "https://host.org/Org/Repo.git"),
    # Gitlab HTTPS
    ("https://host.org/Org/Repo.git", "ssh", "git@host.org:Org/Repo.git"),
    ("https://host.org/Org/Repo.git", "https", "https://host.org/Org/Repo.git"),
    ("https://host.org/Org/Group/Repo.git", "ssh", "git@host.org:Org/Group/Repo.git"),
    (
        "https://host.org/Org/Group/Repo/blob/master/file.py",
        "ssh",
        "git@host.org:Org/Group/Repo.git/blob/master/file.py",
    ),
)

INVALID_PARSE_URLS = (
    ("SSH Bad Username", "gitx@github.com:Org/Repo.git"),
    ("SSH No Repo", "git@github.com:Org"),
    ("HTTPS No Repo", "https://github.com/Org"),
    ("GIT No Repo", "git://github.com/Org"),
)


class UrlRewriteTestCase(unittest.TestCase):
    def _test_rewrite(self, source, protocol, expected):
        parsed = parse(source)
        self.assertTrue(parsed.valid, "Invalid Url: %s" % source)
        return self.assertEqual(parse(source).format(protocol), expected)

    def test_rewrites(self):
        for data in REWRITE_URLS:
            self._test_rewrite(*data)


# Test Suite
suite = unittest.TestLoader().loadTestsFromTestCase(UrlRewriteTestCase)



from .base import BasePlatform


class BitbucketPlatform(BasePlatform):
    PATTERNS = {
        "https": (
            r"(?P<protocols>(git\+)?(?P<protocol>https))://(?P<_user>.+)@(?P<domain>.+?)"
            r"(?P<pathname>/(?P<owner>.+)/(?P<repo>.+?)(?:\.git)?)$"
        ),
        "ssh": (
            r"(?P<protocols>(git\+)?(?P<protocol>ssh))?(://)?git@(?P<domain>.+?):"
            r"(?P<pathname>(?P<owner>.+)/(?P<repo>.+?)(?:\.git)?)$"
        ),
    }
    FORMATS = {
        "https": r"https://%(owner)s@%(domain)s/%(owner)s/%(repo)s.git",
        "ssh": r"git@%(domain)s:%(owner)s/%(repo)s.git",
    }
    DOMAINS = ("bitbucket.org",)
    DEFAULTS = {"_user": "git"}



from .assembla import AssemblaPlatform
from .base import BasePlatform
from .bitbucket import BitbucketPlatform
from .friendcode import FriendCodePlatform
from .github import GitHubPlatform
from .gitlab import GitLabPlatform

# Supported platforms
PLATFORMS = [
    # name -> Platform object
    ("github", GitHubPlatform()),
    ("bitbucket", BitbucketPlatform()),
    ("friendcode", FriendCodePlatform()),
    ("assembla", AssemblaPlatform()),
    ("gitlab", GitLabPlatform()),
    # Match url
    ("base", BasePlatform()),
]



from .base import BasePlatform


class FriendCodePlatform(BasePlatform):
    DOMAINS = ("friendco.de",)
    PATTERNS = {
        "https": (
            r"(?P<protocols>(git\+)?(?P<protocol>https))://(?P<domain>.+?)/"
            r"(?P<pathname>(?P<owner>.+)@user/(?P<repo>.+)).git"
        ),
    }
    FORMATS = {
        "https": r"https://%(domain)s/%(owner)s@user/%(repo)s.git",
    }



import itertools
import re


class BasePlatform:
    FORMATS = {
        "ssh": r"(?P<protocols>(git\+)?(?P<protocol>ssh))?(://)?%(_user)s@%(host)s:%(repo)s.git",
        "http": r"(?P<protocols>(git\+)?(?P<protocol>http))://%(host)s/%(repo)s.git",
        "https": r"(?P<protocols>(git\+)?(?P<protocol>https))://%(host)s/%(repo)s.git",
        "git": r"(?P<protocols>(?P<protocol>git))://%(host)s/%(repo)s.git",
    }

    PATTERNS = {
        "ssh": r"(?P<_user>.+)@(?P<domain>[^/]+?):(?P<repo>.+).git",
        "http": r"http://(?P<domain>[^/]+?)/(?P<repo>.+).git",
        "https": r"https://(?P<domain>[^/]+?)/(?P<repo>.+).git",
        "git": r"git://(?P<domain>[^/]+?)/(?P<repo>.+).git",
    }

    # None means it matches all domains
    DOMAINS = None
    SKIP_DOMAINS = None
    DEFAULTS = {}

    def __init__(self):
        # Precompile PATTERNS
        self.COMPILED_PATTERNS = {proto: re.compile(regex, re.IGNORECASE) for proto, regex in self.PATTERNS.items()}

        # Supported protocols
        self.PROTOCOLS = self.PATTERNS.keys()

        if self.__class__ == BasePlatform:
            sub = [subclass.SKIP_DOMAINS for subclass in self.__class__.__subclasses__() if subclass.SKIP_DOMAINS]
            if sub:
                self.SKIP_DOMAINS = list(itertools.chain.from_iterable(sub))

    @staticmethod
    def clean_data(data):
        data["path"] = ""
        data["branch"] = ""
        data["protocols"] = list(filter(lambda x: x, data["protocols"].split("+")))
        data["pathname"] = data["pathname"].strip(":")
        return data



from .base import BasePlatform


class GitLabPlatform(BasePlatform):
    PATTERNS = {
        "https": (
            r"(?P<protocols>(git\+)?(?P<protocol>https))://(?P<domain>.+?)(?P<port>:[0-9]+)?"
            r"(?P<pathname>/(?P<owner>[^/]+?)/"
            r"(?P<groups_path>.*?)?(?(groups_path)/)?(?P<repo>[^/]+?)(?:\.git)?"
            r"(?P<path_raw>(/blob/|/-/tree/).+)?)$"
        ),
        "ssh": (
            r"(?P<protocols>(git\+)?(?P<protocol>ssh))?(://)?git@(?P<domain>.+?):(?P<port>[0-9]+)?(?(port))?"
            r"(?P<pathname>/?(?P<owner>[^/]+)/"
            r"(?P<groups_path>.*?)?(?(groups_path)/)?(?P<repo>[^/]+?)(?:\.git)?"
            r"(?P<path_raw>(/blob/|/-/tree/).+)?)$"
        ),
        "git": (
            r"(?P<protocols>(?P<protocol>git))://(?P<domain>.+?):(?P<port>[0-9]+)?(?(port))?"
            r"(?P<pathname>/?(?P<owner>[^/]+)/"
            r"(?P<groups_path>.*?)?(?(groups_path)/)?(?P<repo>[^/]+?)(?:\.git)?"
            r"(?P<path_raw>(/blob/|/-/tree/).+)?)$"
        ),
    }
    FORMATS = {
        "https": r"https://%(domain)s/%(owner)s/%(groups_slash)s%(repo)s.git%(path_raw)s",
        "ssh": r"git@%(domain)s:%(port_slash)s%(owner)s/%(groups_slash)s%(repo)s.git%(path_raw)s",
        "git": r"git://%(domain)s%(port)s/%(owner)s/%(groups_slash)s%(repo)s.git%(path_raw)s",
    }
    SKIP_DOMAINS = (
        "github.com",
        "gist.github.com",
    )
    DEFAULTS = {"_user": "git", "port": ""}

    @staticmethod
    def clean_data(data):
        data = BasePlatform.clean_data(data)
        if data["path_raw"].startswith("/blob/"):
            data["path"] = data["path_raw"].replace("/blob/", "")
        if data["path_raw"].startswith("/-/tree/"):
            data["branch"] = data["path_raw"].replace("/-/tree/", "")
        return data



from .base import BasePlatform


class AssemblaPlatform(BasePlatform):
    DOMAINS = ("git.assembla.com",)
    PATTERNS = {
        "ssh": r"(?P<protocols>(git\+)?(?P<protocol>ssh))?(://)?git@(?P<domain>.+?):(?P<pathname>(?P<repo>.+)).git",
        "git": r"(?P<protocols>(?P<protocol>git))://(?P<domain>.+?)/(?P<pathname>(?P<repo>.+)).git",
    }
    FORMATS = {
        "ssh": r"git@%(domain)s:%(repo)s.git",
        "git": r"git://%(domain)s/%(repo)s.git",
    }
    DEFAULTS = {"_user": "git"}



import unittest

from giturlparse import parse

# Test data
VALID_PARSE_URLS = (
    # Valid SSH, HTTPS, GIT
    (
        "SSH",
        (
            "git@github.com:Org/Repo.git",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "Org/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": [],
                "github": True,
                "bitbucket": False,
                "assembla": False,
            },
        ),
    ),
    (
        "HTTPS",
        (
            "git+https://github.com/Org/Repo.git",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org/Repo.git",
                "branch": "",
                "protocol": "https",
                "protocols": ["git", "https"],
                "github": True,
                "bitbucket": False,
                "assembla": False,
            },
        ),
    ),
    (
        "HTTPS",
        (
            "https://github.com/foo-bar/xpwn",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "foo-bar",
                "repo": "xpwn",
                "name": "xpwn",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/foo-bar/xpwn",
                "branch": "",
                "protocol": "https",
                "protocols": ["https"],
                "github": True,
            },
        ),
    ),
    (
        "GIT",
        (
            "git://github.com/Org/Repo.git",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org/Repo.git",
                "branch": "",
                "protocol": "git",
                "protocols": ["git"],
                "github": True,
            },
        ),
    ),
    (
        "GIT",
        (
            "git://github.com/foo-bar/xpwn",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "foo-bar",
                "repo": "xpwn",
                "name": "xpwn",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/foo-bar/xpwn",
                "branch": "",
                "protocol": "git",
                "protocols": ["git"],
                "github": True,
            },
        ),
    ),
    (
        "SSH",
        (
            "ssh://git@github.com/Org/Repo.git",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": ["ssh"],
                "github": True,
            },
        ),
    ),
    # BitBucket
    (
        "SSH",
        (
            "git@bitbucket.org:Org/Repo.git",
            {
                "host": "bitbucket.org",
                "resource": "bitbucket.org",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "Org/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": [],
                "platform": "bitbucket",
            },
        ),
    ),
    # Gitlab
    (
        "SSH",
        (
            "git@host.org:9999/Org/Repo.git",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "9999",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": [],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "SSH",
        (
            "git@host.org:9999/Org-hyphen/Repo.git",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "9999",
                "owner": "Org-hyphen",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org-hyphen/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": [],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "SSH",
        (
            "git@host.org:Org/Repo.git",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "Org/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": [],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "SSH",
        (
            "ssh://git@host.org:9999/Org/Repo.git",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "9999",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org/Repo.git",
                "branch": "",
                "protocol": "ssh",
                "protocols": ["ssh"],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "HTTPS",
        (
            "https://host.org/Org/Repo.git",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": [],
                "path": "",
                "path_raw": "",
                "pathname": "/Org/Repo.git",
                "branch": "",
                "protocol": "https",
                "protocols": ["https"],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "HTTPS",
        (
            "https://github.com/nephila/giturlparse/blob/master/giturlparse/github.py",
            {
                "host": "github.com",
                "resource": "github.com",
                "port": "",
                "user": "git",
                "owner": "nephila",
                "repo": "giturlparse",
                "name": "giturlparse",
                "groups": [],
                "path": "master/giturlparse/github.py",
                "path_raw": "/blob/master/giturlparse/github.py",
                "pathname": "/nephila/giturlparse/blob/master/giturlparse/github.py",
                "branch": "",
                "protocol": "https",
                "protocols": ["https"],
                "platform": "github",
            },
        ),
    ),
    (
        "HTTPS",
        (
            "https://github.com/nephila/giturlparse/tree/feature/py37",
            {
                "host": "github.com",
                "resource": "github.com",
                "user": "git",
                "port": "",
                "owner": "nephila",
                "repo": "giturlparse",
                "name": "giturlparse",
                "groups": [],
                "path": "",
                "path_raw": "/tree/feature/py37",
                "pathname": "/nephila/giturlparse/tree/feature/py37",
                "branch": "feature/py37",
                "protocol": "https",
                "protocols": ["https"],
                "platform": "github",
            },
        ),
    ),
    (
        "HTTPS",
        (
            "https://gitlab.com/nephila/giturlparse/blob/master/giturlparse/github.py",
            {
                "host": "gitlab.com",
                "resource": "gitlab.com",
                "user": "git",
                "port": "",
                "owner": "nephila",
                "repo": "giturlparse",
                "name": "giturlparse",
                "groups": [],
                "path": "master/giturlparse/github.py",
                "path_raw": "/blob/master/giturlparse/github.py",
                "pathname": "/nephila/giturlparse/blob/master/giturlparse/github.py",
                "branch": "",
                "protocol": "https",
                "protocols": ["https"],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "HTTPS",
        (
            "https://gitlab.com/nephila/group2/third-group/giturlparse/blob/master/" "giturlparse/platforms/github.py",
            {
                "host": "gitlab.com",
                "resource": "gitlab.com",
                "user": "git",
                "port": "",
                "owner": "nephila",
                "repo": "giturlparse",
                "name": "giturlparse",
                "groups": ["group2", "third-group"],
                "path": "master/giturlparse/platforms/github.py",
                "path_raw": "/blob/master/giturlparse/platforms/github.py",
                "pathname": "/nephila/group2/third-group/giturlparse/blob/master/" "giturlparse/platforms/github.py",
                "branch": "",
                "protocol": "https",
                "protocols": ["https"],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "SSH",
        (
            "git@host.org:9999/Org/Group/subGroup/Repo.git/blob/master/giturlparse/github.py",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "9999",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": ["Group", "subGroup"],
                "path": "master/giturlparse/github.py",
                "path_raw": "/blob/master/giturlparse/github.py",
                "pathname": "/Org/Group/subGroup/Repo.git/blob/master/giturlparse/github.py",
                "branch": "",
                "protocol": "ssh",
                "protocols": [],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "GIT",
        (
            "git://host.org:9999/Org/Group/subGroup/Repo.git/blob/master/giturlparse/github.py",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "9999",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": ["Group", "subGroup"],
                "path": "master/giturlparse/github.py",
                "path_raw": "/blob/master/giturlparse/github.py",
                "pathname": "/Org/Group/subGroup/Repo.git/blob/master/giturlparse/github.py",
                "branch": "",
                "protocol": "git",
                "protocols": ["git"],
                "platform": "gitlab",
            },
        ),
    ),
    (
        "GIT",
        (
            "git://host.org:9999/Org/Group/subGroup/Repo.git/-/tree/feature/custom-branch",
            {
                "host": "host.org",
                "resource": "host.org",
                "user": "git",
                "port": "9999",
                "owner": "Org",
                "repo": "Repo",
                "name": "Repo",
                "groups": ["Group", "subGroup"],
                "path": "",
                "path_raw": "/-/tree/feature/custom-branch",
                "pathname": "/Org/Group/subGroup/Repo.git/-/tree/feature/custom-branch",
                "branch": "feature/custom-branch",
                "protocol": "git",
                "protocols": ["git"],
                "platform": "gitlab",
            },
        ),
    ),
)

INVALID_PARSE_URLS = (
    ("SSH No Username", "@github.com:Org/Repo.git"),
    ("SSH No Repo", "git@github.com:Org"),
    ("HTTPS No Repo", "https://github.com/Org"),
    ("GIT No Repo", "git://github.com/Org"),
)


# Here's our "unit tests".
class UrlParseTestCase(unittest.TestCase):
    def _test_valid(self, url, expected):
        p = parse(url)
        self.assertTrue(p.valid, "%s is not a valid URL" % url)
        for k, v in expected.items():
            attr_v = getattr(p, k)
            self.assertEqual(attr_v, v, "[{}] Property '{}' should be '{}' but is '{}'".format(url, k, v, attr_v))

    def test_valid_urls(self):
        for _test_type, data in VALID_PARSE_URLS:
            self._test_valid(*data)

    def _test_invalid(self, url):
        p = parse(url)
        self.assertFalse(p.valid)

    def test_invalid_urls(self):
        for _problem, url in INVALID_PARSE_URLS:
            self._test_invalid(url)


# Test Suite
suite = unittest.TestLoader().loadTestsFromTestCase(UrlParseTestCase)



from collections import defaultdict

from .platforms import PLATFORMS

SUPPORTED_ATTRIBUTES = (
    "domain",
    "repo",
    "owner",
    "path_raw",
    "groups_path",
    "_user",
    "port",
    "url",
    "platform",
    "protocol",
)


def parse(url, check_domain=True):
    # Values are None by default
    parsed_info = defaultdict(lambda: None)
    parsed_info["port"] = ""
    parsed_info["path_raw"] = ""
    parsed_info["groups_path"] = ""

    # Defaults to all attributes
    map(parsed_info.setdefault, SUPPORTED_ATTRIBUTES)

    for name, platform in PLATFORMS:
        for protocol, regex in platform.COMPILED_PATTERNS.items():
            # print(name, protocol, regex)
            # Match current regex against URL
            match = regex.match(url)

            # Skip if not matched
            if not match:
                # print("[%s] URL: %s dit not match %s" % (name, url, regex.pattern))
                continue

            # Skip if domain is bad
            domain = match.group("domain")
            # print('[%s] DOMAIN = %s' % (url, domain,))
            if check_domain:
                if platform.DOMAINS and not (domain in platform.DOMAINS):
                    continue
                if platform.SKIP_DOMAINS and domain in platform.SKIP_DOMAINS:
                    continue

            # add in platform defaults
            parsed_info.update(platform.DEFAULTS)

            # Get matches as dictionary
            matches = platform.clean_data(match.groupdict(default=""))

            # Update info with matches
            parsed_info.update(matches)

            # Update info with platform info
            parsed_info.update(
                {
                    "url": url,
                    "platform": name,
                    "protocol": protocol,
                }
            )
            return parsed_info

    # Empty if none matched
    return parsed_info



from .parser import parse as _parse
from .result import GitUrlParsed

__author__ = "Iacopo Spalletti"
__email__ = "i.spalletti@nephila.it"
__version__ = "0.10.0"


def parse(url, check_domain=True):
    return GitUrlParsed(_parse(url, check_domain))


def validate(url, check_domain=True):
    return parse(url, check_domain).valid
